# Note: this file was auto-generated by Make::Build::Convert 0.20

use 5.005;
use strict;
use warnings;

use Module::Build;

my $build = Module::Build->new
  (
   module_name => 'Config::Inetd',
   dist_author => 'Steven Schubiger <schubiger@cpan.org>',
   dist_version_from => 'lib/Config/Inetd.pm',
   requires => { 
                 'Carp' => 0,
                 'Fcntl' => 0,
		 'Tie::File' => 0, 
               },
   build_requires => { 'Test::More' => 0 },
   recommends => { 
                   'Test::Pod' => '1.14',
		   'Test::Pod::Coverage' => '1.04',
		 },
   license => 'perl',
   create_readme => 1,
   create_makefile_pl => 'traditional',
  );
  
$build->create_build_script;
